{$A+,B-,D-,L-,M-,O+,Q-,R-,S-,X+,Y-}
(*
   TChkUpdate 1.0

     "Lets go get the net!"

  --------  --------  --------  --------  --------  --------  --------  --------

  Contents:
     TChkUpdate    - Component for checking wether an update of an application
                   is available on your homepage.

  --------  --------  --------  --------  --------  --------  --------  --------
     Copyrights (C) 1997 ved T Skovmand Eriksen
     email: TSEriksen@cyberdude.com
     ALL RIGHTS RESERVED
  --------  --------  --------  --------  --------  --------  --------  --------

  This source is freeware and may be used as follows:

     * In development of in-house applications which will not be published.

     * In development of freeware applications PROVIDED that credit is given
       to me in the applications aboutbox or helpfile.

     * In development of shareware and commercial applications PROVIDED that
       I, Troels S Eriksen, recieve a free unlimited copy of all versions of
       said application AND that credits are given to me in the applications
       aboutbox or helpfile.

     * In printed or electronic form (fx. in a book) or in a source-library
       (fx. on CD) PROVIDED that I, Troels S Eriksen, recieve a royality-free
       unlimited copy of said book or library.

     * In any other way PROVIDED that I, Troels S Eriksen, give my explicit
       permission to the suggested use.

  By using this source you do acknowledge that the above restrictions apply to
  you and your use of this component and that you will confirm by them.

  --------  --------  --------  --------  --------  --------  --------  --------

  Revision history:
    TChkUpdate
      01    - Component created.

  --------  --------  --------  --------  --------  --------  --------  --------

  TChkUpdate Properties:
     URL                URL to an internet document containing the revision
                        comment to check for
     Homepage           The URL to the main homepage of your company or
                        application. (Used by the OnFound method)
     Revision           Current revision.

  TChkUpdate Events
     OnFound            Triggered if the revision comment is greather than
                        the current revision. Use the Launch value to decide
                        wether the end-users default browser should open the
                        homepage.
     OnNotFound         Triggered if the above is not the true.

  TChkUpdate Methods:
     Execute            Connects to the url and checks the revision. Returns
                        TRUE if connection to the internet was established and
                        the URL found and FALSE if connection could not be
                        established or the URL was not found.

  --------  --------  --------  --------  --------  --------  --------  --------

  Requirements:
     wininet.dll        Windows 32bit Internet API library must be present
                        and working.

  --------  --------  --------  --------  --------  --------  --------  --------

  How to use:
      First, create an webpage for your application. Somewhere within the
      first kilobyte of this homepage, add a revision comment - like this:
      "<!--rev #-->" .  Everytime you update your app, increase the rev
      number (the #) on this webpage.

      Drop a TChkUpdate component on your main form, enter the webpages url in
      the URL property and the current revision number in the Revision
      property.

      Add some code to the OnFound / OnNotFound properties to inform the
      end-user wether theres and update available or not. If you set the
      variable Launch to true, the users browser will automatically show
      the homepage URL. (Would be nice to ask the user permission first)

      Then add some way that the user may call the Execute method (fx. a
      button). Check the return value and inform the use if an internet
      connection could not be made.
*)


unit Updater;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;


type
  TUpdateEvent = procedure (Sender:TObject;var Launch:boolean) of object;

type
  TChkUpdate = class(TComponent)
  private
    { Private declarations }
    fURL         : string;
    fHomepage    : string;
    fRevision    : dword;
    fOnFound     : TUpdateEvent;
    fOnNotFound  : TNotifyEvent;
  protected
  public
    { Methods }
    function    Execute:boolean;
    constructor Create(AnOwner:TComponent); override;
  published
    { Properties }
    property URL        : string  read fURL write fURL;
    property Homepage   : string  read fHomepage write fHomepage;
    property Revision   : integer read fRevision write fRevision;
    { Events }
    property OnFound    : TUpdateEvent read fOnFound write fOnFound;
    property OnNotFound : TNotifyEvent read fOnNotFound write fOnNotFound;
  end;

procedure Register;

implementation
uses
  Wininet,ShellApi;

procedure Register;
begin
  RegisterComponents('Samples', [TChkUpdate]);
end;

function TChkUpdate.Execute:Boolean;
type
  tbuffer    = array[0..1023] of char;
  pbuffer    = ^TBuffer;
var
  Launch     : boolean;
  buffer     : pBuffer;
  a,b        : integer;
  ihConnect,
  ihSession,
  iDocument  : HINTERNET;
  pBuf       : pointer;
begin
  result:=false;
  // Forbind til internettet
  ihConnect:=InternetOpen('Modzilla/2.0 (Updater/Eriksen)',
    LOCAL_INTERNET_ACCESS, '', '', 0);
  // bn dokument
  iDocument:=InternetOpenURL(ihConnect,
    pChar(fURL), NIL, -1, INTERNET_FLAG_RELOAD or
    INTERNET_FLAG_DONT_CACHE or INTERNET_FLAG_RAW_DATA, 0);
  // Ls frste kilobyte af dokument
  if iDocument<>NIL then try
    New(buffer);
    InternetReadFile(iDocument, Buffer, sizeof(TBuffer), a);
    Buffer^[a]:=#0;
    // Indgr kommentaren "rev" i frste kb ?
    a:=pos('<!--rev',AnsiLowercase(string(buffer^)));
    if a>0 then begin
      // Ja - afls revisionsnummer - det skal flge efter rev teksten
      b:=a+6;
      while buffer[b] in ['0'..'9',' '] do inc(b);
      // Returner Sand hvis hjere rev nr findes p net
      result:=StrToIntDef(Trim(Copy(string(Buffer^),a+7,b-a-7)),0)>fRevision;
    end;
    // Vis hndelse
    try
      case result of
        true : if assigned(fOnFound) then begin
          fOnFound(Self,Launch);
          If (launch) and (trim(fHomepage)>'') then
            ShellExecute(Application.Handle,'open',pChar(fHomepage),'','',
              SW_SHOWNORMAL);
        end;
        false: if assigned(fOnNotFound) then
          fOnNotFound(Self);
      end;
    finally
      result:=true;
    end;
  finally
    internetCloseHandle(iDocument);
    Dispose(buffer);
    InternetCloseHandle(ihConnect);
  end;
end;

constructor TChkUpdate.Create(AnOwner:TComponent);
begin
  Inherited ;
  URL     :='http://webserver/application/revision.htm';
  Homepage:='http://webserver/application/';
  Revision:=1;
end;

end.



